//
// The Amoeba Generator is a program that computes and visualizes
// amoebas for some complex-valued bivariate expressions.
//
// Copyright (C) 2008 Magnus Leksell and Wojciech Komorowski.
//
// http://amoeba.sajberspejs.com
// Email: amoeba@sajberspejs.com
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef CONFFILE_H_
#define CONFFILE_H_

#include <string>
#include <list>
#include <cln/lfloat.h>
#include <cln/complex.h>
#include <libxml++/libxml++.h>
#include "environment.h"
#include "number.h"
#include "cpolynomial.h"
#include "interval.h"

//
// Class ConfFile
//
class ConfFile : public Environment, protected xmlpp::SaxParser {
  bool onDescription_;
  bool onPowerSeries_;
  bool firstTime_;
  bool verbose_;
  //std::map<int, cln::cl_N> zPoly_;
  //std::map<int, cln::cl_N> wPoly_;

  // Private methods
  cln::cl_LF createLF(const string& x);
  cln::cl_N createComplexLF(const string& x);
  void createTaylor(const cln::cl_N& coeff, const string& fun,
                    const string& var, unsigned int terms);
  void taylorexpSin(const cln::cl_N& coeff, const string& var,
                    unsigned int terms);
  void taylorexpCos(const cln::cl_N& coeff, const string& var,
                    unsigned int terms);
  void taylorexpExp(const cln::cl_N& coeff, const string& var,
                    unsigned int terms);
  bool validate(const string& exp);
  int eval(const string& exp, unsigned int j);
  void createPowerSeries(const cln::cl_N& coeff, const string& var,
                         const string& cexp, const string& vexp,
                         const string& jcoeff, const string& fun);
  void addSeriesToPolyString(const cln::cl_N& coeff, const string& var,
                             const string& cexp, const string& vexp,
                             const string& jcoeff, const string& fun);
  void addToPolyString(const cln::cl_N& coeff, unsigned int zexp,
                       unsigned int wexp);
  void addToPolyString(const cln::cl_N& coeff, const string& fun,
                       const string& var);
  void setAmoeba(const AttributeList& properties);
  void setSettings(const AttributeList& properties);
  void setImage(const AttributeList& properties);
  void setLatex(const AttributeList& properties);
  void setData(const AttributeList& properties);
  void setNPolytope(const AttributeList& properties);
  void setPolynomial(const AttributeList& properties);
  void setPowerSeries(const AttributeList& properties);
  void setTerm(const AttributeList& properties);
  void setInterval(const AttributeList& properties);
  void setMax(const AttributeList& properties);
  void setMin(const AttributeList& properties);

protected:
  // Overrides xmlpp:
  virtual void on_start_document();
  virtual void on_end_document();
  virtual void on_start_element(const Glib::ustring& name,
                                const AttributeList& properties);
  virtual void on_end_element(const Glib::ustring& name);
  virtual void on_characters(const Glib::ustring& characters);
  virtual void on_comment(const Glib::ustring& text);
  virtual void on_warning(const Glib::ustring& text);
  virtual void on_error(const Glib::ustring& text);
  virtual void on_fatal_error(const Glib::ustring& text);

public:
  ConfFile(bool verbose = false);
  ~ConfFile();
  // Overrides Environment:
  //const CPolynomial& getPolynomial();
  void parseFile(const string& filename);
};

#endif /* CONFFILE_H_ */
