//
// The Amoeba Generator is a program that computes and visualizes
// amoebas for some complex-valued bivariate expressions.
//
// Copyright (C) 2008 Magnus Leksell and Wojciech Komorowski.
//
// http://amoeba.sajberspejs.com
// Email: amoeba@sajberspejs.com
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "environment.h"

//
// Constructor
//
Environment::Environment() {
  // Set some default values
  polyString_ = "p(z,w)=";
  imageWidth_ = 640;
  imageHeight_ = 480;
  amoebaName_ = "Amoeba";
  description_ = "An amoeba";
  imageFileName_ = "amoeba.png";
  latexFileName_ = "amoeba.tex";
  dataFileName_ = "amoeba.dat";
  newtonPolytopeFileName_ = "npolytope.png";
  precision_ = 50;
  maxError_ = Number("1.0e-30");
  maxProcesses_ = 2;
  minLnW_ = -5.0;
  maxLnW_ = 20.0;
  minLnZ_ = -10.0;
  maxLnZ_ = 10.0;
  powerSeriesRange_.first = 0;  // from
  powerSeriesRange_.second = 0; // to
  wantsAxes_ = false;
}

//
// Destructor
//
Environment::~Environment() {
}

const CPolynomial& Environment::getPolynomial() {
  return poly_;
}

const string& Environment::getPolynomialString() const {
  return polyString_;
}

const list<AbstractInterval*>& Environment::getIntervals() const {
  return intervals_;
}

int Environment::getImageWidth() const {
  return imageWidth_;
}

int Environment::getImageHeight() const {
  return imageHeight_;
}

const string& Environment::getDescription() const {
  return description_;
}

const string& Environment::getAmoebaName() const {
  return amoebaName_;
}

const string& Environment::getImageFileName() const {
  return imageFileName_;
}

const string& Environment::getLatexFileName() const {
  return latexFileName_;
}

const string& Environment::getDataFileName() const {
  return dataFileName_;
}

const string& Environment::getNewtonPolytopeFileName() const {
  return newtonPolytopeFileName_;
}

unsigned int Environment::getPrecision() const {
  return precision_;
}

const Number& Environment::getMaxError() const {
  return maxError_;
}

unsigned int Environment::getMaxProcesses() const {
  return maxProcesses_;
}

double Environment::getMinLnZ() const {
  return minLnZ_;
}

double Environment::getMaxLnZ() const {
  return maxLnZ_;
}

double Environment::getMinLnW() const {
  return minLnW_;
}

double Environment::getMaxLnW() const {
  return maxLnW_;
}

const UIntPair& Environment::getPowerSeriesRange() const {
  return powerSeriesRange_;
}

bool Environment::wantsAxes() const {
  return wantsAxes_;
}

ostream& operator << (ostream& os, const Environment& env) {
  os << "Amoeba name: " << env.amoebaName_ << endl
     << "Image file name: " << env.imageFileName_ << endl
     << "Image width: " << env.imageWidth_ << endl
     << "Image height: " << env.imageHeight_ << endl
     << "Image coord axes: " << (env.wantsAxes_?"yes":"no") << endl
     << "Description: " << env.description_ << endl
     << "LaTeX file name: " << env.latexFileName_ << endl
     << "Data file name: " << env.dataFileName_ << endl
     << "Newton polytope file name: " << env.newtonPolytopeFileName_ << endl
     << "Precision: " << env.precision_ << endl
     << "Max error: " << env.maxError_ << endl
     << "Processes: " << env.maxProcesses_ << endl
     << "Min ln w: " << env.minLnW_ << endl
     << "Max ln w: " << env.maxLnW_ << endl
     << "Min ln z: " << env.minLnZ_ << endl
     << "Max ln z: " << env.maxLnZ_ << endl
     << "Polynomial: " << env.polyString_ << endl;
}

/*
void Environment::parseFile(const string& filename) {
  // You must overide this method
}
*/
