//
// The Amoeba Generator is a program that computes and visualizes
// amoebas for some complex-valued bivariate expressions.
//
// Copyright (C) 2008 Magnus Leksell and Wojciech Komorowski.
//
// http://amoeba.sajberspejs.com
// Email: amoeba@sajberspejs.com
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef ENVIRONMENT_H_
#define ENVIRONMENT_H_

#include <string>
#include <list>
#include "number.h"
#include "cpolynomial.h"
#include "interval.h"

typedef std::pair<unsigned int, unsigned int> UIntPair;

//
// Class Environment
//
class Environment {
protected:
  CPolynomial poly_;
  string polyString_;
  int imageWidth_;
  int imageHeight_;
  string amoebaName_;
  string description_;
  string imageFileName_;
  string latexFileName_;
  string dataFileName_;
  string newtonPolytopeFileName_;
  list<AbstractInterval*> intervals_;
  unsigned int precision_;
  Number maxError_;
  unsigned int maxProcesses_;
  double minLnW_;
  double maxLnW_;
  double minLnZ_;
  double maxLnZ_;
  UIntPair powerSeriesRange_;
  bool wantsAxes_;

public:
  Environment();
  virtual ~Environment();
  virtual const CPolynomial& getPolynomial();
  virtual const string& getPolynomialString() const;
  virtual const list<AbstractInterval*>& getIntervals() const;
  virtual int getImageWidth() const;
  virtual int getImageHeight() const;
  virtual const string& getAmoebaName() const;
  virtual const string& getDescription() const;
  virtual const string& getImageFileName() const;
  virtual const string& getLatexFileName() const;
  virtual const string& getDataFileName() const;
  virtual const string& getNewtonPolytopeFileName() const;
  virtual unsigned int getPrecision() const;
  virtual const Number& getMaxError() const;
  virtual unsigned int getMaxProcesses() const;
  virtual double getMinLnZ() const;
  virtual double getMaxLnZ() const;
  virtual double getMinLnW() const;
  virtual double getMaxLnW() const;
  virtual const UIntPair& getPowerSeriesRange() const;
  virtual bool wantsAxes() const;
  virtual void parseFile(const string& filename) = 0;
  friend ostream& operator << (ostream& os, const Environment& env);
};

#endif /* ENVIRONMENT_H_ */
