//
// The Amoeba Generator is a program that computes and visualizes
// amoebas for some complex-valued bivariate expressions.
//
// Copyright (C) 2008 Magnus Leksell and Wojciech Komorowski.
//
// http://amoeba.sajberspejs.com
// Email: amoeba@sajberspejs.com
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include <gd.h>
#include <stdexcept>
#include "image.h"

using namespace std;

void ImageProducer::write(const char* filename, const Points& points,
  int width, int height) throw(runtime_error, out_of_range) {

  // Validate image size
  if (width < 1)
    throw out_of_range("Image width < 1");

  if (height < 1)
    throw out_of_range("Image height < 1");

  gdImagePtr im = gdImageCreate(width, height);
  int white = gdImageColorAllocate(im, 255, 255, 255); // Background color white
  int black = gdImageColorAllocate(im, 0, 0, 0);

  Points::const_iterator iter = points.begin();
  for (; iter != points.end(); ++iter) {
    Key key = (*iter).first;
    int x = key.first;
    int y = key.second;
    if (x >=0 && x < width && y >= 0 && y < height) {
      //cout << "setPixel (" << x << "," << y << ")" << endl;
      gdImageSetPixel(im, x, y, black);
    }
  }

  FILE* pngout = fopen(filename, "wb");
  if (pngout == NULL)
    throw runtime_error("Could not save data to file!");

  gdImagePng(im, pngout);
  gdImageDestroy(im);
}

