//
// The Amoeba Generator is a program that computes and visualizes
// amoebas for some complex-valued bivariate expressions.
//
// Copyright (C) 2008 Magnus Leksell and Wojciech Komorowski.
//
// http://amoeba.sajberspejs.com
// Email: amoeba@sajberspejs.com
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef INTERVAL_H_
#define INTERVAL_H_

#include <queue>
#include <ostream>
#include "complex.h"
#include "number.h"

using namespace std;

// This class represents a region in the complex plane from which
// substitute values will be taken to replace one of the variables
// in the bivariate polynomial p(z, w), so that univariate
// polynomials can be created as a step in plotting the amoeba

class AbstractInterval {
protected:
  // If the region is a rectangular
  Complex from_;
  Complex to_;
  
  // If the region is a disk centered at the origin
  Number minRadius_;
  Number maxRadius_;
  Number step_;   // step length between the radiuses
  int points_;    // amount of points on the circumference of each circle
public:
  // If the region is rectangular, this is the upper left corner
  virtual const Complex& from() const { return from_; }

  // If the region is rectangular, this is the lower right corner
  virtual const Complex& to() const { return to_; }
  
  // If the region is a disk centered at the origin, this is the
  // minimum radius of the circle-set that models the disk
  virtual const Number& minRadius() const { return minRadius_; }
  
  // And this is the corresponding maximum radius
  virtual const Number& maxRadius() const { return maxRadius_; }

  // This is the difference between each circle radius
  virtual const Number& step() const { return step_; }

  // Amount of points on the circumference of each circle
  virtual int points() const { return points_; }

  virtual ostream& print(ostream& os) const = 0;
  friend ostream& operator<<(ostream& os, const AbstractInterval& i) {
    return i.print(os);
  }
};

#endif /* INTERVAL_H_ */
