//
// The Amoeba Generator is a program that computes and visualizes
// amoebas for some complex-valued bivariate expressions.
//
// Copyright (C) 2008 Magnus Leksell and Wojciech Komorowski.
//
// http://amoeba.sajberspejs.com
// Email: amoeba@sajberspejs.com
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include <iostream>
#include <fstream>
#include <stdexcept>
#include "latex.h"

using namespace std;

void LatexProducer::write(const Environment& env) throw(runtime_error) {
  string fileName = env.getLatexFileName();
  extern const char* argp_program_version; // quick and dirty solution!

  if (fileName == "") // Empty string -- no file name given:
    return;           // silently skip this.

  ofstream os(fileName.c_str());
  if (!os)
    throw runtime_error("Could not create LaTeX file " + fileName);

  // Produce info about the amoeba
  os << "%" << endl
     << "% Created by " << argp_program_version << endl
     << "% http://amoeba.sajberspejs.com" << endl
     << "%" << endl
     << "\\documentclass[a4paper]{article}" << endl
     //<< "\\usepackage[english]{babel}" << endl
     //<< "\\usepackage[utf8]{inputenc}" << endl
     << "\\usepackage[T1]{fontenc}" << endl
     << "\\usepackage{a4}" << endl
     << "\\usepackage{amsmath}" << endl
     << "\\usepackage{subfigure}" << endl
     //<< "\\usepackage[pdftex]{graphicx}" << endl
     << "\\usepackage{graphicx}" << endl
     << "\\title{" << env.getAmoebaName() << "}" << endl
     << "\\author{}" << endl
     << "\\date{}" << endl
     << "\\begin{document}" << endl
     << "\\maketitle" << endl
     << "\\thispagestyle{empty}" << endl
     << "\\begin{figure}[!hbt]" << endl
     << "\\centering" << endl
     << "\\subfigure[$\\mathcal{N}_p$]{"
     << "\\includegraphics[width=.20\\textwidth]{"
     << env.getNewtonPolytopeFileName() << "}}" << endl
     << "\\hfill" << endl
     << "\\subfigure[$\\mathcal{A}_p$]{"
     << "\\includegraphics[width=.75\\textwidth]{"
     << env.getImageFileName() << "}}" << endl
     << "\\caption{Newton polytope and amoeba for $p(z,w)$}"
     //<< env.getPolynomialString() << "$}" << endl
     << "\\end{figure}" << endl
     << "\\[" << env.getPolynomialString() << "\\]" << endl
     << "\\paragraph{Description}" << env.getDescription() << "\\\\" << endl;

     // TODO:
     // Precision/settings used.
     // ...

  /*const list<AbstractInterval*> intervals = env.getIntervals();
  list<AbstractInterval*>::const_iterator iter = intervals.begin();
  int count = 1;
  for (; iter != intervals.end(); ++iter) {
    os << "Interval " << count++ << "(" << intervals.size() << "): "
       << **iter << "\\\\" << endl;
  }*/

  os << "\\vfill" << endl
     << "\\noindent Created by " << argp_program_version << "\\\\" << endl
     << "\\texttt{http://amoeba.sajberspejs.com}" << endl
     << "\\end{document}" << endl;

  os.close();

  cout << "Saved LaTeX in " << fileName << endl;
}

