//
// The Amoeba Generator is a program that computes and visualizes
// amoebas for some complex-valued bivariate expressions.
//
// Copyright (C) 2008 Magnus Leksell and Wojciech Komorowski.
//
// http://amoeba.sajberspejs.com
// Email: amoeba@sajberspejs.com
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include <map>
#include <list>
#include <string>
#include <iostream>
#include <fstream>
#include <stdexcept>
#include "listplotter.h"
#include "point.h"
#include "color.h"
#include "image.h"

using namespace std;

//
// Method plotList
//
void ListPlotter::plotList(const char* infile, const char* outfile, int width,
                           int height) throw(runtime_error) {
  // Treat input from stdin as special case
  string str = infile;
  if (str == "-") {
    plotList(cin, outfile, width, height);
    return;
  }

  ifstream ifile(infile);
  if (!ifile)
    throw runtime_error("Could not open input file: " + str);

  cout << "Reading list of x, y values from " << infile << endl;
  try {
    plotList(ifile, outfile, width, height);
  }
  catch (const exception& e) {
    ifile.close();
    throw runtime_error(e.what());
  }

  ifile.close();
}

//
// Private method plotList
//
void ListPlotter::plotList(istream& is, const char* outfile, int width,
                           int height) throw(runtime_error) {
  list<pair<double, double> > values;
  double minX, maxX, minY, maxY;
  minX = maxX = minY = maxY = 0;
  double xpos, ypos;
    
  while (!is.eof() ) {
    is >> xpos;
    is >> ypos;
    if (xpos < minX) minX = xpos;
    if (xpos > maxX) maxX = xpos;
    if (ypos < minY) minY = ypos;
    if (ypos > maxY) maxY = ypos;
    values.push_back(pair<double, double>(xpos, ypos));
  }

  double dx = (maxX - minX)/(double)width;
  double dy = (minY - maxY)/(double)height;
  int intx, inty;
  map<Key, Color> points;
  list<pair<double, double> >::const_iterator iter;
  for (iter = values.begin(); iter != values.end(); iter++) {
    xpos = (*iter).first;
    ypos = (*iter).second;
    intx = (int)((xpos - minX)/dx);
    inty = (int)((ypos - maxY)/dy);
    Key key(intx, inty);
    points[key] = DEFAULT_COLOR;
  }

  ImageProducer::write(outfile, points, width, height);
}

