#include <iostream>
#include <fstream>
#include <list>
#include <cln/abort.h>
#include <cln/io.h>

#include <cln/number.h>
#include <cln/complex.h>
#include <cln/complex_io.h>
#include <cln/real.h>
#include <cln/real_io.h>
#include <cln/rational.h>
#include <cln/rational_io.h>
#include <cln/integer.h>
#include <cln/integer_io.h>
#include <cln/float.h>
#include <cln/sfloat.h>
#include <cln/ffloat.h>
#include <cln/dfloat.h>
#include <cln/lfloat.h>
#include <cln/univpoly_complex.h>
#include <cln/input.h>

#include <cln/float_io.h>
#include <cln/sfloat_io.h>
#include <cln/ffloat_io.h>
#include <cln/dfloat_io.h>
#include <cln/lfloat_io.h>
#include <cln/complex_io.h>

#include "CPSolver.h"

using namespace std;
using namespace cln;

int main(int argc, char** argv)
{
  if (argc != 2) {
    cout << "Usage: " << argv[0] << " filename" << endl;
    return 0;
  }
  
  ifstream polyFile;
  polyFile.open(argv[1]);
  if (!polyFile.is_open()) {
    cerr << "Error: Could not open " << argv[1] << endl;
    return 1;
  }
  
  int prec;
  polyFile >> prec;
  float_format_t precision = float_format(prec);
  cl_F zero = cl_float(0, precision);
  cl_F one = cl_float(1, precision);
  
  cl_R maxError;
  polyFile >> maxError;
  
  cl_univpoly_complex_ring R = find_univpoly_ring(cl_C_ring);
  cl_UP_N p = R->create(0);
  p.set_coeff(0, complex(one, zero));

  while (! polyFile.eof() ) {
    string line;
    getline(polyFile, line);
    if (line == "") continue;
    cl_N c_tmp(line.c_str());
    cl_UP_N p_tmp = R->create(1);
    p_tmp.set_coeff(1, complex(one, zero));
    p_tmp.set_coeff(0, -c_tmp);
    p_tmp.finalize();
    p = p*p_tmp;
  }
  
  cout << "p(x) = " << p << endl;
  
  CPSolver solver;
  list<cl_N> p1roots = solver.getRoots(p, precision, maxError);
  list<cl_N> p1roots2 = p1roots;

  cout << "roots are:" << endl;
  int i = 1;
  while (!p1roots.empty()) {
  	cl_N tmp = p1roots.front();
  	p1roots.pop_front();
        cout << "r" << i << ": " << endl;
        cout << realpart(tmp) << endl;
        cout << imagpart(tmp) << "i" << endl;
        i++;
  }

  cout << "------------------------------------------------------------------------------" << endl;
  cout << "Verifying roots..." << endl;

  i = 1;
  while (!p1roots2.empty()) {
    cl_N root = p1roots2.front();
    p1roots2.pop_front();
    cl_N eval = p(root);
    cout << "|p(r" << i++ << ")| = " << abs(eval) << endl;
  }

  return 0;
}
